/* DMA Controller
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef DMA_INTERNAL_H
#define DMA_INTERNAL_H

/** \cond INTERNAL */

/** \addtogroup dma
 * @{
 */

/** \file dma_internal.h
 * \brief internal structures for Direct Memory Access Controller
 * \note not to be used by other components
 */

/** \brief dma channel structure */
struct DmaChannel {
	/** \brief drcm register */
	uint32_t drcm;
	/** \brief interrupt callback */
	void (*cbDmaInterrupt)(uint8_t channel, uint32_t remainingLength,
		void * data);
	/** \brief user defined data for callback */
	void * data;
};

/** \brief Get the address portion of a DDADR field \a adr */
#define DDADR_ADDR(adr) ((struct DmaDescriptor*)(((uint32_t)(adr)) & ~0xf))

/** \brief Get the flag portion of a DDADR field \a adr */
#define DDADR_FLAGS(adr) (((uint32_t)(adr)) & 0xf)

/** @} */

/** \endcond */

#endif

