/* Image definitions, detection and booting
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef IMAGE_H
#define IMAGE_H

#include <inttypes.h>
#include <stdlib.h>

/** \file image.h
 * \brief All image related stuff, like validity checking, retrieving header
 *  information and finally booting
 */

enum ImageVersionFlag {
	IMAGE_VERSION_OLD,
	IMAGE_VERSION_NEW
};

struct Image {
	const void * lma;
	enum ImageVersionFlag versionFlag;
	const void * vma;
	void (*entry)();
	uint32_t size;
	uint32_t version;
	const char * description;
	bool bootable;
};

void imageRead(const void * lma, size_t maxsize, struct Image * img);

void imageReadDefault(struct Image * img);

__attribute__((noreturn))
void imageBoot(const struct Image * img);

#endif

