/* SDRAM activation
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

It contains ideas taken from the blob project, licenced under the GPL.

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <os_arm_bare_registers_def.h>
#include <inttypes.h>
#include "flash_internal.h"

__attribute__((noinline,no_instrument_function,section(".ramtext"),long_call))
static void initSyncFlash() {
	asm volatile(
		"b 1f\n\t"
		".align 5\n\t"
		"1:\n\t"
		"str %[SXCNFG_VAL], [%[SXCNFG_REG]]\n\t"
		"strh %[FLASH_WRITE_VAL], [%[FLASH_SYNC_VAL]]\n\t"
		"strh %[FLASH_WCONF_VAL], [%[FLASH_SYNC_VAL]]\n\t"
		"strh %[FLASH_READ_VAL], [%[FLASH_BASE]]\n\t"
		"ldrh ip, [%[FLASH_BASE]]\n\t"
		"nop\n\t"
		"nop\n\t"
		"nop\n\t"
		::	[SXCNFG_VAL] "r" (SXCNFG_SXEN0 | SXCNFG_SXCL0(4) | SXCNFG_SXSET |
				SXCNFG_SXTP0(3)),
			[SXCNFG_REG] "r" (&SXCNFG),
			[FLASH_SYNC_VAL] "r" (0x25C3 << 1),
			[FLASH_WRITE_VAL] "r" (0x0060),
			[FLASH_WCONF_VAL] "r" (0x0003),
			[FLASH_READ_VAL] "r" (CMD_READ_ARRAY),
			[FLASH_BASE] "r" (FLASH_BASE_ADDR)
		: "ip"
		);
}
#if 0
__attribute__((noinline,no_instrument_function,section(".ramtext"),long_call))
static void flash_async() {
	asm volatile(
		"b 1f\n\t"
		".align 5\n\t"
		"1:\n\t"
		"str %[SXCNFG_VAL], [%[SXCNFG_REG]]\n\t"
		"strh %[FLASH_WRITE_VAL], [%[FLASH_SYNC_VAL]]\n\t"
		"strh %[FLASH_WCONF_VAL], [%[FLASH_SYNC_VAL]]\n\t"
		"strh %[FLASH_READ_VAL], [%[FLASH_BASE]]\n\t"
		"ldrh %[tmp], [%[FLASH_BASE]]\n\t"
		"nop\n\t"
		"nop\n\t"
		"nop\n\t"
		::	[SXCNFG_VAL] "r" (SXCNFG_SXCL0(4) | SXCNFG_SXSET |
				SXCNFG_SXTP0(3)),
			[SXCNFG_REG] "r" (&SXCNFG),
			[FLASH_SYNC_VAL] "r" ((0x25c3 | (1 << 15)) << 1),
			[FLASH_WRITE_VAL] "r" (0x0060),
			[FLASH_WCONF_VAL] "r" (0x0003),
			[FLASH_READ_VAL] "r" (CMD_READ_ARRAY),
			[FLASH_BASE] "r" (FLASH_BASE_ADDR),
			[tmp] "r" (0)
		);
}

__attribute__((noinline,no_instrument_function,section(".ramtext"),long_call))
static void flash_sync() {
	uint16_t pcr;
	
	asm volatile(
		"b 1f\n\t"
		".align 5\n\t"
		"1:\n\t"
		"str %[SXCNFG_VAL], [%[SXCNFG_REG]]\n\t"
		"strh %[FLASH_WRITE_VAL], [%[FLASH_SYNC_VAL]]\n\t"
		"strh %[FLASH_WCONF_VAL], [%[FLASH_SYNC_VAL]]\n\t"
		"strh %[FLASH_READ_VAL], [%[FLASH_BASE]]\n\t"
		"ldrh %[tmp], [%[FLASH_BASE]]\n\t"
		"nop\n\t"
		"nop\n\t"
		"nop\n\t"
		::	[SXCNFG_VAL] "r" (SXCNFG_SXEN0 | SXCNFG_SXCL0(4) | SXCNFG_SXSET |
				SXCNFG_SXTP0(3)),
			[SXCNFG_REG] "r" (&SXCNFG),
			[FLASH_SYNC_VAL] "r" ((0x25c3) << 1),
			[FLASH_WRITE_VAL] "r" (0x0060),
			[FLASH_WCONF_VAL] "r" (0x0003),
			[FLASH_READ_VAL] "r" (CMD_READ_ARRAY),
			[FLASH_BASE] "r" (FLASH_BASE_ADDR),
			[tmp] "r" (0)
		);
}
#endif

/** initialize the memory controller
 * this code was initialy written by Philip Buonadonna, Robbie Adler */
__attribute__((no_instrument_function,section(".start")))
void sdramEnable() {
	/* SXSTACK: nCS<0> contains stacked flash */
	SA1110 = SA1110_SXSTACK(1);
	
	/* Set Bank 0 (Flash Memory)
		- Bus Width: 16 Bit
		- Streaming Behavoiur
		- ROM Type: Synchronous flash */
	MSC0 |= MSC_RBW024 | MSC_RBUFF024 | MSC_RT024(0);
	
	/* Set Bank 3 and 5: Bus Width: 16 Bit */
	MSC1 |= MSC_RBW024;
	MSC2 |= MSC_RBW024;
	
	/* Expansion Memory (PC Card/CF) Bus Configuration register:
		no PC Card inserted and 1 socket */
	MECR = 0;
	
	/*
	MCMEM0 PC Card Interface Common Memory Space Socket 0 Timing Config Reg
	MCMEM1 ...
	MCATT0 PC Card Interface Attribute Space Socket 0 Timing Config Reg
	MCATT1 ...
	MCIO0  PC Card Interface I/O Space Socket 0 Timing Config Reg
	MCIO1  ...
	*/
	
	
	/* Fly-By DMA DVAL */
	//FLYCNFG
	
	
	/* SDRAM Configuration register:
		DWID0: 16 Bit Bus Width
		DCAC0: 9 column address bits
		DRAC0: 13 row address bits
		DNB0:  4 internal SDRAM banks
		DTC0:  RAS Precharge (tRP): 3 clks, CAS Latency (CL): 3,
			RAS-to CAS Delay (tRCD): 3 clks, RAS pulse width (tRAS): 7 clks,
			Row Cycle Time (tRC): 11 clks
			This means we have a 3-3-3-7 SDRAM
		SETALWAYS: must be set (reserved)
		STACK0: Address Multiplexing: SDRAM Address is placed on MA<24:23,13:1>
	*/
	MDCNFG = (MDCNFG_DTC2(0x3) |MDCNFG_STACK0 | MDCNFG_SETALWAYS |
		  MDCNFG_DTC0(0x3) | MDCNFG_DNB0 | MDCNFG_DRAC0(0x2) | 
		  MDCNFG_DCAC0(0x1) | MDCNFG_DWID0);
	
	
	/* Refresh Configuration:
		Reconfigure Refresh Interval (DRI) to 799 clks 
		Deassert all clocks */
	MDREFR = (MDREFR & ~0xfff) | MDREFR_DRI(0x18); 
	
	//PXA27x MemController 5th tier initialization.See 6.4.10 for details
	/* put flash in sync mode */
	initSyncFlash();
	
	/* Reconfigure Refresh:
		Disable free running clocks, enable SDRAM Clock Pin 1 and 0,
		run them the half memory frequency
	*/
	MDREFR = (MDREFR & ~(MDREFR_K0FREE | MDREFR_K1FREE | MDREFR_K2FREE)) | 
	  (MDREFR_K1RUN | MDREFR_K1DB2 | MDREFR_K0DB2 | MDREFR_K0RUN); 
	/* disable self-refresh */
	MDREFR &= ~MDREFR_SLFRSH;
	
	/* enable SDCKE (Clock Enable Pin 1) */
	MDREFR |= MDREFR_E1PIN;
	
	/* ensure a stable clock by waiting about 250us */
	uint32_t start = OSCR0;
	while(OSCR0 - start < 800);
	
	
	/* Enable refresh on SDRAM by writing to it */
	asm volatile (".rept 8\n\tstr %0, [%0]\n\t.endr" :: "r" (0xa0000000));
	
	
	/* Enable SDRAM */
	MDCNFG |= MDCNFG_DE0;
	
	/* Trigger MRS command */
	MDMRS = 0;
	
	/* Enable auto power down mode */
	MDREFR |= MDREFR_APD;
}

__attribute__((no_instrument_function,section(".start")))
void sdramDisable() {
	/* TODO */
	
	/* disable ram */
	//MDCNFG &= ~MDCNFG_DE0;
	
	/* disable clock 1 */
	//MDREFR &= ~MDREFR_E1PIN;
	
	//wait();
	
	/* activate self refresh */
	//MDREFR |= MDREFR_SLFRSH;
	
	/* disable bank 1 */
	//MDREFR &= ~(MDREFR_K1RUN | MDREFR_K1DB2);
	
	//wait();
}

