#include "os_arm_bare_memory.h"
#include "inttypes.h"

int16_t lock_counter=0;


/*
this funtion is intended to lock critical sections
upon first call the function disables all interrupts
*/
EXTERN_C void __malloc_lock(void){
#ifdef SENF_DEBUG_INT_ENABLED
	if(lock_counter==0)IMOTE_BLUE_ON;
#endif
	asm volatile (
		/* get processor status in ip */
		"mrs ip,CPSR\n\t"
		/* disable interrupts by setting the flag */
		"orr ip,ip,%[mask]\n\t"
		/* write back new processor status */
		"msr CPSR_c,ip"
		:
		: [mask] "i" (ARM_CPSR_INT_MASK)
		: "ip", "memory", "cc"
		);
	lock_counter++;
}

/*
this function is intended to unlock a set lock
the lock is set in __malloc_lock
if no more lock left, last lock is released by reenabling interrupts
this function is safe for more unlocking than locking calls
*/
EXTERN_C void __malloc_unlock(void){
	lock_counter--;
	if(lock_counter <= 0){
		lock_counter = 0;
#ifdef SENF_DEBUG_INT_ENABLED
		IMOTE_BLUE_OFF;
#endif
		asm volatile (
			/* get processor status in ip */
			"mrs ip,CPSR\n\t"
			/* enable interrupts by clearing the flag */
			"bic ip, ip, %[mask]\n\t"
			/* write back new processor status */
			"msr CPSR_c, ip"
			:
			: [mask] "i" (ARM_CPSR_BIT_I)
			: "ip", "memory", "cc"
		);
	}
}
