/* Simple tool to inject boot vectors
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

int main(int argc, char *argv[]) {
	int fd;
	struct stat fdstat;
	unsigned long addr;
	char * data;
	
	/* check argument count */
	if (argc != 3) {
		fprintf(stderr, "Usage: %s <filename> <address>\n", argv[0]);
		return 1;
	}
	
	/* parse address */
	char * endptr;
	addr = strtoul(argv[2], &endptr, 0);
	if (*endptr != '\0') {
		fprintf(stderr, "Could not parse '%s' as digit\n", argv[2]);
		return 1;
	}
	
	/* open file */
	printf("Opening file '%s'\n", argv[1]);
	fd = open(argv[1], O_RDWR);
	if (fd == -1) {
		fprintf(stderr, "Could not open file '%s': %s\n", argv[1],
			strerror(errno));
		return 1;
	}
	
	/* get file size */
	if (fstat(fd, &fdstat) == -1) {
		fprintf(stderr, "Could not read filesize: %s\n", strerror(errno));
		close(fd);
		return 1;
	}
	
	/* test address against file size */
	if (addr + 32 > fdstat.st_size) {
		fprintf(stderr, "Reading Address 0x%08lx for 32 bits exceeds "
			"filesize %d", addr, (int)fdstat.st_size);
		close(fd);
		return 1;
	}
	
	/* mmap file */
	data = mmap(NULL, fdstat.st_size, PROT_READ | PROT_WRITE, MAP_SHARED,
		fd, 0x0);
	if (!data) {
		fprintf(stderr, "Could not mmap file: %s\n", strerror(errno));
		close(fd);
		return 1;
	}
	
	/* compare vectors */
	if (!memcmp(data + addr, data, 32)) {
		/* match: nothing to do */
		printf("Vectors match, all done\n");
	} else {
		/* no match: inject */
		printf("Vectors do not match, injecting boot vector\n");
		memmove(data + addr, data, 32);
	}
	
	/* unmap */
	munmap(data, fdstat.st_size);
	
	/* close file */
	close(fd);
	
	return 0;
}

